#############################################################################
#
# Authors: Michel F. SANNER, Ruth HUEY and Garrett M. MORRIS
# Reimplemented from Babel v1.6 from Pat Walters and Math Stahl
#
# Copyright: M. Sanner TSRI 2000
#
#
#  Cobalt parameters added by Alex L. Perryman 8/24/2011 (due to user request)
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/PyBabel/gasteiger.py,v 1.8 2011/09/02 18:41:39 rhuey Exp $
#
# $Id: gasteiger.py,v 1.8 2011/09/02 18:41:39 rhuey Exp $
#
#

"""
This file implements the gasteiger class.

Before this gasteiger object can be used, atoms must have been assigned
a type see (AtomHybridization in types.py).

reimplmentation of Babel1.6 in Python by Michel Sanner April 2000
Original code by W. Patrick Walters and Matthew T. Stahl 
"""



import math
from atomTypes import TypeConverter

###Partial equilization of orbital electronegativity, PEOP
###PEOP method to calculate a, b and c
###  from  J. Gasteiger and M. Marsili, Tetrahedron 36, 3219 (1980)
###
###energy parameters
###
###for neutral atoms: 
###  from J. Hinze and H.H. Jaffe, J. Am. Chem. Soc. 84, 540 (1962)
###     I0 Valence State Ionization Potentials 
###     E0 Valence State Electron Affinities 
###
###for positive ions: 
###  from J. Hinze and H.H. Jaffe, J. Phys. Chem. 67, 1501 (1963)
###     I+ Valence State Ionization Potentials 
###     E+ Valence State Electron Affinities 
###
### orbital ids used:
###     te tetrahedral, sp3
###     tr trigonal, sp2
###     di digonal, sp

energyParms = {}

#elemDict['sp2'] = hybridD = {}
#elemDict['sp2'] = hybridD = {}
#hybridD['I0'] =
#hybridD['E0'] =
#hybridD['I+'] =
#hybridD['E+'] =

#Carbon
energyParms['C'] = {}
energyParms['C']['sp3'] = hybridD ={}     
hybridD['I0'] = 14.61                #tetetete
hybridD['E0'] =  1.34                #tetetete
hybridD['I+'] = 26.71                #tetete, sigma
hybridD['E+'] = 11.37                #tetete, sigma
energyParms['C']['sp2'] = hybridD ={}     
hybridD['I0'] = 15.62                #trtrtr pi, sigma
hybridD['E0'] =  1.95                #trtrtr pi, sigma
hybridD['I+'] = 27.36                #trtr pi, sigma
hybridD['E+'] = 11.91                #trtr pi, sigma
#hybridD['I+'] = 23.68                #trtr pi, pi
#hybridD['E+'] = 10.45                #trtr pi, pi
energyParms['C']['sp'] = hybridD ={}     
hybridD['I0'] = 17.42                #didi pi pi, sigma
hybridD['E0'] =  3.34                #didi pi pi, sigma
hybridD['I+'] = 28.16                #di pi pi, sigma
hybridD['E+'] = 12.96                #di pi pi, sigma
#hybridD['I+'] = 29.85                #didi pi, sigma
#hybridD['E+'] = 13.29                #didi pi, sigma
#hybridD['I+'] = 23.86                #didi pi, pi
#hybridD['E+'] =  9.83                #didi pi, pi

#Nitrogen
energyParms['N'] = {}
energyParms['N']['sp3'] = hybridD ={}     
hybridD['I0'] = 18.93                #te2tetete
hybridD['E0'] =  4.15                #te2tetete
hybridD['I+'] = 33.29                #tetetete, sigma
hybridD['E+'] = 14.14                #tetetete, sigma
energyParms['N']['sp2'] = hybridD ={}     
hybridD['I0'] = 20.60                #tr2trtr pi, sigma
hybridD['E0'] =  5.14                #tr2trtr pi, sigma
hybridD['I+'] = 34.62                #trtrtr pi, sigma
hybridD['E+'] = 15.09                #trtrtr pi, sigma
#hybridD['I+'] = 28.71                #trtrtr pi, pi
#hybridD['E+'] = 11.96                #trtrtr pi, pi
energyParms['N']['sp'] = hybridD ={}     
hybridD['I0'] = 23.91                #di2di pi pi, sigma
hybridD['E0'] =  7.45                #di2di pi pi, sigma
hybridD['I+'] = 37.00                #didi pi pi, sigma
hybridD['E+'] = 17.24                #didi pi pi, sigma
#hybridD['I+'] = 28.70                #didi pi pi, pi
#hybridD['E+'] = 12.06                #didi pi pi, pi

#Oxygen
energyParms['O'] = {}
energyParms['O']['te'] = hybridD ={}     
hybridD['I0'] = 24.39                #te2te2tete
hybridD['E0'] =  6.11                #te2te2tete
hybridD['I+'] = 40.31                #te2tetete, sigma
hybridD['E+'] = 18.70                #te2tetete, sigma
energyParms['O']['sppp'] = hybridD ={}     
hybridD['I0'] = 17.28                #s2 p2 p p
hybridD['E0'] =  2.01                #s2 p2 p p
hybridD['I+'] = 34.15                #s2 p p p, p 
hybridD['E+'] = 14.61                #s2 p p p, p
#hybridization adjusted to an angle of 106deg [methanol]
#106 = 81% of 109.5 [te] + 19% of 90deg [sppp]
energyParms['O']['sp3'] = hybridD ={}     
prop = 0.81
z = energyParms['O']['te']
y = energyParms['O']['sppp']
hybridD['I0'] = prop * z['I0'] + (1.0 - prop) * y['I0']                # methanol
hybridD['E0'] = prop * z['E0'] + (1.0 - prop) * y['E0']                # methanol
hybridD['I+'] = prop * z['I+'] + (1.0 - prop) * y['I+']                # methanol
hybridD['E+'] = prop * z['E+'] + (1.0 - prop) * y['E+']                # methanol
energyParms['O']['sp2'] = hybridD ={}     
hybridD['I0'] = 26.65                #tr2 tr2 tr pi, sigma
hybridD['E0'] =  7.49                #tr2 tr2 tr pi, sigma
hybridD['I+'] = 42.49                #tr2 tr tr pi, sigma
hybridD['E+'] = 20.15                #tr2 tr tr pi, sigma

## new stuff
#Beryllium (2)
energyParms['Be'] = elemDict = {}
elemDict['sp3'] = hybridD = {}
hybridD['I0'] =  7.18           #tete
hybridD['E0'] =  0.51           #tete
hybridD['I+'] = 18.21           #s
hybridD['E+'] =  9.32           #s
#hybridD['I+'] = 14.25           #p
#hybridD['E+'] =  5.32           #p
elemDict['sp2'] = hybridD = {}
hybridD['I0'] =  7.38           #tr pi, sigma
hybridD['E0'] =  0.63           #tr pi, sigma
hybridD['I+'] = 18.21           #s
hybridD['E+'] =  9.32           #s
#hybridD['I+'] = 14.25           #p
#hybridD['E+'] =  5.32           #p

#Sulfur (2)
energyParms['S'] = elemDict = {}
elemDict['sp3'] = hybridD = {}
hybridD['I0'] = 15.50           #te2te2tete
hybridD['E0'] =  4.77           #te2te2tete
hybridD['I+'] = 27.65           #te2tetete, sigma
hybridD['E+'] = 13.64           #te2tetete, sigma
elemDict['sp2'] = hybridD = {}
hybridD['I0'] = 16.27           #tr2 tr tr pi2
hybridD['E0'] =  5.49           #tr2 tr tr pi2
hybridD['I+'] = 28.51           #tr tr tr pi2, sigma
hybridD['E+'] = 14.33           #tr tr tr pi2, sigma

#Phosphorus (3)
energyParms['P'] = elemDict = {}
elemDict['sp3'] = hybridD = {}
hybridD['I0'] = 14.57           #te2tetete
hybridD['E0'] =  3.24           #te2tetete
hybridD['I+'] = 24.10           #tetetete, sigma
hybridD['E+'] = 12.09           #tetetete, sigma
elemDict['sp2'] = hybridD = {}
hybridD['I0'] = 15.59           #tr2 tr tr pi, sigma
hybridD['E0'] =  3.74           #tr2 tr tr pi, sigma
hybridD['I+'] = 25.14           #tr tr tr pi, sigma
hybridD['E+'] = 12.72           #tr tr tr pi, sigma

#Aluminium (3)
energyParms['Al'] = elemDict = {}
elemDict['sp3'] = hybridD = {}
hybridD['I0'] =  8.17           #tetete
hybridD['E0'] =  2.58           #tetete
hybridD['I+'] = 15.75           #tete, sigma
hybridD['E+'] =  6.64           #tete, sigma
elemDict['sp2'] = hybridD = {}
hybridD['I0'] =  8.65           #trtr pi, sigma
hybridD['E0'] =  2.94           #trtr pi, sigma
hybridD['I+'] = 16.28           #tr pi, sigma
hybridD['E+'] =  6.74           #tr pi, sigma

#Boron (3)
energyParms['B'] = elemDict = {}
elemDict['sp3'] = hybridD = {}
hybridD['I0'] = 10.43           #tetete
hybridD['E0'] =  1.53           #tetete
hybridD['I+'] = 20.93           #tete, sigma
hybridD['E+'] =  7.88           #tete, sigma
elemDict['sp2'] = hybridD = {}
hybridD['I0'] = 10.97           #trtr pi, sigma
hybridD['E0'] =  1.87           #trtr pi, sigma
hybridD['I+'] = 21.08           #tr pi, sigma
hybridD['E+'] =  8.02           #tr pi, sigma

#Silicon (4)
energyParms['Si'] = {}
energyParms['Si']['sp3'] = hybridD ={}     
hybridD['I0'] = 11.82                #tetetete
hybridD['E0'] =  2.78                #tetetete
hybridD['I+'] = 18.97                #tetete, sigma
hybridD['E+'] = 10.08                #tetete, sigma
energyParms['Si']['sp2'] = hybridD ={}     
hybridD['I0'] = 12.61                #trtrtr pi, sigma
hybridD['E0'] =  3.20                #trtrtr pi, sigma
hybridD['I+'] = 19.62                #trtr pi, sigma
hybridD['E+'] = 10.57                #trtr pi, sigma
#hybridD['I+'] = 16.53                #trtr pi, pi
#hybridD['E+'] = 9.54                #trtr pi, pi
energyParms['Si']['sp'] = hybridD ={}     
hybridD['I0'] = 14.06                #didi pi pi, sigma
hybridD['E0'] =  4.07                #didi pi pi, sigma
hybridD['I+'] = 20.62                #di pi pi, sigma
hybridD['E+'] = 11.56                #di pi pi, sigma
#hybridD['I0'] =  9.18                #didi pi pi, pi
#hybridD['E0'] =  2.20                #didi pi pi, pi
#hybridD['I+'] = 16.50                #didi pi, pi
#hybridD['E+'] =  8.60                #didi pi, pi

#Magnesium
energyParms['Mg'] = {}
energyParms['Mg']['sp3'] = hybridD ={}     
hybridD['I0'] =  6.28                #tete
hybridD['E0'] =  0.32                #tete
hybridD['I+'] = 15.03                #s, s
hybridD['E+'] =  7.64                #s, s
energyParms['Mg']['sp2'] = hybridD ={}     
hybridD['I0'] =  6.75                #tr pi, sigma
hybridD['E0'] =  0.38                #tr pi, sigma
hybridD['I+'] = 15.03                #s, s
hybridD['E+'] =  7.64                #s, s
energyParms['Mg']['sp'] = hybridD ={}     
hybridD['I0'] =  7.30                #di pi, sigma
hybridD['E0'] =  0.78                #di pi, sigma
hybridD['I+'] = 15.03                #s, s
hybridD['E+'] =  7.64                #s, s


#Cobalt
energyParms['Co'] = {}
energyParms['Co']['sp3'] = hybridD ={}     
hybridD['I0'] =  7.88                #tete
hybridD['E0'] =  0.66                #tete
hybridD['I+'] = 17.08                #s, s
hybridD['E+'] =  7.88                #s, s
#Note: these values for Co are actually NOT atom-type specific; data = on element Co
#these values for Co added by Alex L. Perryman (due to a request by Khia-Yu Ku)
#must find "promotion energies" for Co before making atom-type specific corrections

class Gasteiger:
    """ """
    
    def __init__(self):
        """ """
        self.setup_sigma_params()


    def setup_sigma_params(self):
        """ """

        self.par = {
            "H" : [  7.17,  6.24, -0.56, 0.0 ],
            "C3": [  7.98,  9.18,  1.88, 0.0 ],
            "C2": [  8.79,  9.32,  1.51, 0.0 ],
            "C1": [ 10.39,  9.45,  0.73, 0.0 ],
            "N3": [ 11.54, 10.82,  1.36, 0.0 ],
            "N2": [ 12.87, 11.15,  0.85, 0.0 ],
            "N1": [ 15.68, 11.70, -0.27, 0.0 ],
            "O3": [ 14.18, 12.92,  1.39, 0.0 ],
            "O2": [ 17.07, 13.79,  0.47, 0.0 ],
            "F" : [ 14.66, 13.85,  2.31, 0.0 ],
            "Cl": [ 11.00,  9.69,  1.35, 0.0 ],
            "Br": [ 10.08,  8.47,  1.16, 0.0 ],
            "I" : [  9.90,  7.96,  0.96, 0.0 ],
            "CO": [  4.27,  6.08,  2.13, 0.0 ],
            "Co": [  4.27,  6.08,  2.13, 0.0 ],
            "S3": [ 10.14,  9.13,  1.38, 0.0 ]
            }

        #add values compute from energyParms information
        #NB: this overwrites specified entries BUT the numbers are
        #    virtually the same...
        for element, elementD in energyParms.items():
            for hybrid, hybridD in elementD.items():
                I0 = hybridD['I0']
                E0 = hybridD['E0']
                IPLUS = hybridD['I+']
                EPLUS = hybridD['E+']
                a = 0.5 * (I0 + E0)
                a = round(a, 3)
                b = 0.25 * (IPLUS + EPLUS - E0)
                b = round(b, 3)
                c = 0.25 * (IPLUS - 2 * I0 + EPLUS - E0)
                c = round(c, 3)
                if hybrid=='sp':
                    key = element + '1'
                elif hybrid not in ['sp2', 'sp3']:
                    continue
                else:
                    key = element + hybrid[-1]
                self.par[key] = [a, b, c, 0.0]
                
        for p in self.par.values():
            p[3] = p[0] + p[1] + p[2]


    def lookup_sigma_params(self, atoms):
        """ """
  
        converter = TypeConverter("MAP")
        for a in atoms:
            type = converter.convert( a.babel_type )
            if type[:2] in self.par.keys():
                a._gast_par = self.par[type]
            else:
                print "Sorry, there are no Gasteiger parameters available for atom %s"%a.full_name()
                a._gast_par = [ 0.0, 0.0, 0.0, 1.0 ]


    def calc_sigma_charges(self, atoms):
        """ """

        # set formal charges
        for a in atoms:
            a.gast_charge = 0.0
            if a.babel_atomic_number==6: # carbons
                if a.babel_type=="C+": a.gast_charge = 1.0
            elif a.babel_atomic_number==7: # nitrogens
                if len(a.bonds)==4: a.gast_charge = 1.0
                if a.babel_type == "N3+": a.gast_charge = 1.0
            elif a.babel_atomic_number==8: # oxygens
                if a.babel_type=="O-": a.gast_charge = -0.5
            a._xx = a._gast_par[0]

        z1 = 1.0
        cycle = 0

        while (1):

            z1 = z1 * 0.5
            d1 = 0.0

            for a in atoms:
                if a._gast_par[3] != 1.0:
                    q1 = a.gast_charge
                    for b in a.bonds:
                        a2 = b.atom1
                        if a2==a: a2 = b.atom2
                        if a2._gast_par[3] != 1.0:
                            hybridD = a2._gast_par[3]
                            if a2._xx > a._xx: hybridD = a._gast_par[3]
                            if a.babel_type == 'H': hybridD = 20.02
                            if a2.babel_type == 'H': hybridD = 20.02
                            a.gast_charge = a.gast_charge + \
                                            (a2._xx - a._xx)/hybridD*z1

                    q1 = math.fabs(a.gast_charge - q1)
                    if q1 > d1: d1 = q1

            if d1 >= 0.001:
                for a in atoms:
                    a._xx = a._gast_par[0] + a._gast_par[1] * a.gast_charge + \
                           a._gast_par[2] * a.gast_charge * a.gast_charge
            cycle = cycle+1
            if d1<=0.001 or cycle>5: break
 

    def compute(self, atoms):
        """compute(atoms) compute gasteiger charges for a set of atoms"""
        
        self.lookup_sigma_params(atoms)
        self.calc_sigma_charges(atoms)

        # cleanup
        for a in atoms:
            delattr(a,'_gast_par')
            delattr(a,'_xx')



if __name__=="__main__":
#    import pdb, sys
#    from MolKit.pdbParser import NewPdbParser

#    parser = NewPdbParser("/tsri/pdb/struct/%s.pdb"%sys.argv[1])
#    mols = parser.parse()
#    mol = mols[0]
#    mol.buildBondsByDistance()
#    allAtoms = mol.chains.residues.atoms

#    print "assigning atom types"
#    from atomTypes import AtomHybridization
#    babel = AtomHybridization()
#    babel.assignHybridization(allAtoms)

#    print "computing charges"
#    Gast = Gasteiger()
#    Gast.compute(allAtoms)

#    print allAtoms.babel_type[:20]
#    print allAtoms.gast_charge[:20]

#    f = open("result.charges", "w")
#    for i in range(len(allAtoms)):
#        a = allAtoms[i]
#        f.write("%2d %4s %10.4f\n"%(i+1, a.babel_type, a.gast_charge))
#    f.close()
    
    for element, elementD in energyParms.items():
        print 'Element = ', element
        for hybrid, hybridD in elementD.items():
            I0 = hybridD['I0']
            E0 = hybridD['E0']
            IPLUS = hybridD['I+']
            EPLUS = hybridD['E+']
            a = 0.5 * (I0 + E0)
            b = 0.25 * (IPLUS + EPLUS - E0)
            c = 0.25 * (IPLUS - 2 * I0 + EPLUS - E0)
            print '    ',hybrid, ' hybridization:'
            print '        a =', a,
            print 'b =', b,
            print 'c =', c
        print '\n',
            





#!/usr/bin/env python

from distutils.core import setup
from distutils.command.sdist import sdist
from distutils.command.install_data import install_data
from glob import glob
import os

########################################################################
# Had to overwrite the prunrefile_list method of sdist to not
# remove automatically the RCS/CVS directory from the distribution.
########################################################################

class modified_sdist(sdist):
    def prune_file_list(self):
        """
        Prune off branches that might slip into the file list as created
        by 'read_template()', but really don't belong there:
          * the build tree (typically 'build')
          * the release tree itself (only an issue if we ran 'sdist
            previously with --keep-temp, or it aborted)
        """
        build = self.get_finalized_command('build')
        base_dir = self.distribution.get_fullname()
        self.filelist.exclude_pattern(None, prefix=build.build_base)
        self.filelist.exclude_pattern(None, prefix=base_dir)

class modified_install_data(install_data):

    def run(self):
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return install_data.run(self)
########################################################################

# list of the python packages to be included in this distribution.
# sdist doesn't go recursively into subpackages so they need to be
# explicitaly listed.
# From these packages only the python modules will be taken
packages = ['PyBabel', 'PyBabel.Tests']

# list of the python modules not part of a package. Give the path and the
# filename without the extension. i.e you want to add the
# test.py module which is located in MyPack/Tests/ you give
# 'MyPack/Tests/test'
py_modules = []

# list of the files that are not python packages but are included in the
# distribution and need to be installed at the proper place  by distutils.
# The list in MANIFEST.in lists is needed for including those files in
# the distribution, data_files in setup.py is needed to install them
# at the right place.
data_files = []
for dir in ['PyBabel',
            'PyBabel/Tests/Data',
            'PyBabel/CVS', 
            'PyBabel/Tests/CVS', 
            'PyBabel/Tests/Data/CVS']:
    files = []
    for f in glob(os.path.join(dir, '*')):
        if f[-3:] != '.py' and f[-4:-1] != '.py' and os.path.isfile(f):
            files.append(f)
    data_files.append((dir, files))

# description of what is going to be included in the distribution and
# installed.
from version import VERSION
setup (name = 'PyBabel',
       version = VERSION,
       description = "an OpenGL based 3D geometry viewer python package",
       author = 'Molecular Graphics Laboratory',
       author_email = 'sanner@scripps.edu',
       download_url = 'http://www.scripps.edu/~sanner/software/packager.html',
       url = 'http://www.scripps.edu/~sanner/software/index.html',
       packages = packages,
       py_modules = py_modules,
       data_files = data_files,
       cmdclass = {'sdist': modified_sdist,
                   'install_data': modified_install_data
                   },
       )



#############################################################################
#
# Author: Michel F. SANNER
# Reimplemented from Babel v1.6 from Pat Walters and Math Stahl
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/PyBabel/atomTypes.py,v 1.6 2012/10/19 16:09:49 rhuey Exp $
#
# $Id: atomTypes.py,v 1.6 2012/10/19 16:09:49 rhuey Exp $
#

"""
This file implements the AtomHybridization class that can be used to assign
atom types.

example:
    
      >>> atype = AtomHybridization()
      >>> atype.assignHybridization(atoms)

      atoms has to be a list of atom objects
      Atom:
          a.element : atom's chemical element symbol (string)
          a.coords : 3-sequence of floats
          a.bonds : list of Bond objects
      Bond:
          b.atom1 : instance of Atom
          b.atom2 : instance of Atom

      after completion each atom has the following additional members:
          babel_type: string
          babel_atomic_number: int
          babel_organic
      
reimplmentation of Babel1.6 in Python by Michel Sanner April 2000
Original code by W. Patrick Walters and Matthew T. Stahl 
"""



import string

from babelAtomTypes import babel_types
from babelElements import babel_elements
from util import *


# for valence_three
SP3_MAX     = 114.8
MAY_BE_SP2  = 122.0
SP_MIN      = 160.0

# for valence_one
V1_C1_C1_CUTOFF = 1.22
V1_C2_C_CUTOFF  = 1.41
V1_C2_N_CUTOFF  = 1.37

V1_N1_C1_CUTOFF = 1.20 
V1_N3_C_CUTOFF  = 1.38
V1_N3_N3_CUTOFF = 1.43
V1_N3_N2_CUTOFF = 1.41

V1_O2_C2_CUTOFF = 1.30
V1_O2_AS_CUTOFF = 1.685

V1_S2_C2_CUTOFF = 1.76
V1_S2_AS_CUTOFF = 2.11

V2_C3_C_CUTOFF  = 1.53
V2_C3_N_CUTOFF  = 1.46
V2_C3_O_CUTOFF  = 1.44

V2_N2_C_CUTOFF  = 1.38
V2_N2_N_CUTOFF  = 1.32

V2_C2_C_CUTOFF  = 1.42
V2_C2_N_CUTOFF  = 1.41
GEN_C3_C_CUTOFF = 1.45


class AtomHybridization:


    def __init__(self):
        """constructor"""
        self.atoms = None
        
        
    def get_atomic_number(self, name):
        """return the element number for a given name or raises a
        ValueError exception if the element is not known"""
        _name = string.upper(name[0])
        if len(name)>1:
            if not name[1] in string.digits:
                _name = _name + string.lower(name[1])
        if _name in babel_elements.keys():
            return babel_elements[_name]['num'] 
        else:
            raise ValueError( "Could not find atomic number for %s %s"% \
                              (name,_name) )


    def assignHybridization(self, atoms):
        """atoms is a list of objects of type Atom having the following
        members:
        Atom:
            a.element : atom's chemical element symbol (string)
            a.coords : 3-sequence of floats
            a.bonds : list of Bond objects
        Bond:
            b.atom1 : instance of Atom
            b.atom2 : instance of Atom

        after completion each atom has the following additional members:
        babel_type: string
        babel_atomic_number: int
        babel_organic
        """
        
        self.atoms = atoms
        for a in self.atoms:
            a.babel_type = a.element
            a._babel_redo = 0
            a.babel_atomic_number = self.get_atomic_number(a.babel_type)

            if a.babel_type[0] in [ 'C', 'H', 'O', 'N', 'S', 'P' ]:
                a.babel_organic=1
            else: a.babel_organic = 0
            if a.element=='Ca': a.babel_organic = 0
        
        self.phase1()
        self.valence_four()
        self.valence_three()
        self.valence_two()
        self.valence_one()

        self.phase4()
        self.phase5() 
        self.phase6()
        self.check_for_amides()

        # cleanup
        for a in self.atoms:
            delattr(a,'_babel_redo')

        delattr(self,'atoms')


    def count_heavy_atoms(self, atom):
        count = 0
        for b in atom.bonds:
            bonded_atom = b.atom1
            if bonded_atom==atom: bonded_atom=b.atom2
            if bonded_atom.babel_type[0] == 'H': count = count + 1
        return len(atom.bonds) - count


    def count_free_ox(self, atom):
        free_O_count=0
        for b in atom.bonds:
            bonded_atom = b.atom1
            if bonded_atom==atom: bonded_atom=b.atom2
            if bonded_atom.babel_type[0] == 'O' and \
               self.count_heavy_atoms(bonded_atom) == 1:
                free_O_count = free_O_count+1
        return free_O_count

        
    def phase1(self):
        for a in self.atoms:
            if a.babel_type[0] == 'H':
                a.babel_type = 'H'

                if len(a.bonds):
                    k = a.bonds[0].atom1
                    if k==a: k = a.bonds[0].atom2
                    if k.babel_type[0] == 'C' and k.element == 'C':
                        a.babel_type = 'HC'


    def valence_four(self):

      for a in self.atoms:
        if len(a.bonds) == 4 and a.babel_organic:

            if a.babel_type[0] == 'C' and a.element == 'C':
                if a.babel_type=='C': a.babel_type = "C3"

            elif a.babel_type[0] == 'N':
                if self.count_free_ox(a) >= 1: a.babel_type = "Nox"
                else: a.babel_type = "N3+"

            elif a.babel_type[0] == 'P':
                if len(a.babel_type) == 1:
                    count = self.count_free_ox(a)
                    if count >= 2: a.babel_type = "Pac"
                    elif count == 1: a.babel_type = "Pox"
                    else: a.babel_type = "P3+"

            elif a.babel_type[0] == 'S':
                if a.babel_type=='S':
                    count = self.count_free_ox(a)
                    if count >= 3: a.babel_type = "Sac"
                    elif count >= 1: a.babel_type = "Sox"
                    else: a.babel_type = "S"

            elif a.babel_type[0] == 'B':
                count = self.count_free_ox(a)
                if count >= 3: a.babel_type = "Bac"
                if count >= 1: a.babel_type = "Box"
                else: a.babel_type = "B"


    def valence_three(self):

      for a in self.atoms:
          if len(a.bonds) == 3 and a.babel_organic:

              k = a.bonds[0].atom1
              if k==a: k = a.bonds[0].atom2
              l = a.bonds[1].atom1
              if l==a: l = a.bonds[1].atom2
              m = a.bonds[2].atom1
              if m==a: m = a.bonds[2].atom2

              angle1 = bond_angle(k.coords, a.coords, l.coords)
              angle2 = bond_angle(k.coords, a.coords, m.coords)
              angle3 = bond_angle(l.coords, a.coords, m.coords)
              avg_angle = (angle1 + angle2 + angle3)/3

              if a.babel_type[0] =='C' and a.element == 'C':
                  if avg_angle < SP3_MAX: a.babel_type="C3"
                  elif self.count_free_ox(a) >= 2: a.babel_type="Cac"
                  else: a.babel_type = "C2"

              elif a.babel_type[0] =='N':
                  if avg_angle < SP3_MAX: a.babel_type = "N3"
                  elif self.count_free_ox(a) >= 2: a.babel_type = "Ntr"
                  else: a.babel_type = "Npl"

              elif a.babel_type[0] =='B':
                  if self.count_free_ox(a) >= 1: a.babel_type = "Box"
                  else: a.babel_type = "B"

              elif a.babel_type =='S':
                  if self.count_free_ox(a) >= 1: a.babel_type = "Sox"
                  else: a.babel_type = "S3+"      


    def valence_two(self):

      for a in self.atoms:
          if len(a.bonds) == 2 and a.babel_organic:

              k = a.bonds[0].atom1
              if k==a: k = a.bonds[0].atom2
              l = a.bonds[1].atom1
              if l==a: l = a.bonds[1].atom2
              
              if a.coords[0]==l.coords[0] and a.coords[1]==l.coords[1] and a.coords[2]==l.coords[2]:
                  print  a.full_name() +" and " +l.full_name() +" have the same coordinates"
                  
              angle1 = bond_angle(k.coords, a.coords, l.coords)

              if a.babel_type[0] == 'C' and a.element == 'C':
                  if a.babel_type =="C":
                      if angle1 < SP3_MAX:
                          a.babel_type = "C3"
                          a._babel_redo = 1
                      elif angle1 < SP_MIN:
                          a.babel_type = "C2"
                          if angle1 < MAY_BE_SP2:
                              a._babel_redo = 3
                      else: a.babel_type = "C1"

              elif a.babel_type[0] == 'N':
                  if angle1 <= SP3_MAX:
                      a.babel_type = "N3"
                      a._babel_redo = 2
                  elif angle1 <= SP_MIN: a.babel_type = "Npl"
                  else: a.babel_type = "N1"

              elif a.babel_type[0] == 'O':
                a.babel_type = "O3"

              elif a.babel_type[0] == 'S':
                  if a.babel_type =="S": a.babel_type = "S3"


    def valence_one(self):

      for a in self.atoms:

          if len(a.bonds) == 1 and a.babel_organic:
              k = a.bonds[0].atom1
              if k==a: k = a.bonds[0].atom2
              bond_length = distance(a.coords, k.coords)

              if a.babel_type[0] == 'C' and a.element == 'C':
                  if a.babel_type=="C":
                      if k.babel_type[:2]=='C1' and \
                         bond_length <= V1_C1_C1_CUTOFF:
                          a.babel_type = "C1"
                      elif k.babel_type[0] == "C" and \
                           bond_length <= V1_C2_C_CUTOFF:
                          a.babel_type = "C2"
                      else: a.babel_type = "C3"

                  if k.babel_type[0]=="N":
                      if bond_length <= V1_C2_N_CUTOFF: a.babel_type = "C2"
                      else: a.babel_type = "C3"

              if a.babel_type[0] == 'N':
                   if a.babel_type=="N":
                      if k.babel_type[:2]=='C1' and \
                         bond_length <= V1_N1_C1_CUTOFF:
                          a.babel_type = "N1"
                      elif (k.babel_type[:2] == "C2" or \
                            k.babel_type[:2] == "C3") \
                           and bond_length > V1_N3_C_CUTOFF:
                          a.babel_type = "N3"
                      elif a.babel_type[:2]== "N3" and \
                           bond_length > V1_N3_N3_CUTOFF:
                          a.babel_type = "N3"
                      elif a.babel_type[:2]== "Npl" and \
                           bond_length > V1_N3_N2_CUTOFF:
                          a.babel_type = "N3"
                      else:
                          a.babel_type = "Npl"

              if a.babel_type[0] == 'O':
                  if a.babel_type=="O":
                      if k.babel_type in ["Cac", "Pac", "Sac", "Ntr"]:
                          a.babel_type = "O-"
                      elif k.babel_type in ["Nox", "Pox", "Sox"]:
                          a.babel_type = "O2"
                      elif k.babel_type[0] =="C" and \
                           bond_length <= V1_O2_C2_CUTOFF:
                          a.babel_type = "O2"
                          k.babel_type = "C2"
                          k._babel_redo = 0
                      elif k.babel_type=="As" and \
                           bond_length <= V1_O2_AS_CUTOFF:
                          a.babel_type = "O2"
                      else: a.babel_type = "O3"

              if a.babel_type[0] == 'S':
                  if a.babel_type=="S":
                      if k.babel_type[0] =="P": a.babel_type = "S2"
                      elif k.babel_type[0]=="C" and \
                           bond_length <= V1_S2_C2_CUTOFF:
                          a.babel_type = "S2"
                          k.babel_type = "C2"
                          a._babel_redo = 0
                      elif k.babel_type=="As" and \
                           bond_length <= V1_S2_AS_CUTOFF:
                          a.babel_type = "S2"
                      else: a.babel_type = "S3"


    def phase4(self):

      for a in self.atoms:
          if a._babel_redo==1:
              for b in a.bonds:
                  j = b.atom1
                  if j==a: j = b.atom2
                  bond_length = distance(a.coords, j.coords)
                  if bond_length <= V2_C2_C_CUTOFF and j.babel_type[0] == 'C' and j.element == 'C':
                      a.babel_type = "C2"
                  elif bond_length <= V2_C2_N_CUTOFF and j.babel_type[0] =='N':
                      a.babel_type = "C2"

              for b in a.bonds:
                  j = b.atom1
                  if j==a: j = b.atom2
                  bond_length = distance(a.coords, j.coords)
                  if bond_length > V2_C3_C_CUTOFF and j.babel_type[0] == 'C' and j.element == 'C':
                      a.babel_type = "C3"
                  elif bond_length > V2_C3_N_CUTOFF and j.babel_type[0] == 'N':
                      a.babel_type = "C3"
                  elif bond_length > V2_C3_O_CUTOFF and j.babel_type[0] == 'O':
                      a.babel_type = "C3"

          elif a._babel_redo==2:
              for b in a.bonds:
                  j = b.atom1
                  if j==a: j = b.atom2
                  bond_length = distance(a.coords, j.coords)
                  if bond_length <= V2_N2_C_CUTOFF and j.babel_type[0] == 'C' and j.element == 'C':
                      a.babel_type = "Npl"
                  elif bond_length <= V2_N2_N_CUTOFF and j.babel_type[0] =='N':
                      a.babel_type = "Npl"

          elif a._babel_redo==3:
              flag = 0
              for b in a.bonds:
                  j = b.atom1
                  if j==a: j = b.atom2
                  bond_length = distance(a.coords, j.coords)

                  if bond_length <= V2_C2_C_CUTOFF and j.babel_type[0] == 'C' and j.element == 'C':
                      a.babel_type = "C2"
                      flag = 1
                  elif bond_length <= V2_C2_N_CUTOFF and j.babel_type[0] =='N':
                      a.babel_type = "C2"
                      flag = 1

              if flag == 0:
                  for b in a.bonds:
                      j = b.atom1
                      if j==a: j = b.atom2
                      bond_length = distance(a.coords, j.coords)
                      if bond_length > V2_C3_C_CUTOFF and j.babel_type[0]=='C' and j.element == 'C':
                          a.babel_type = "C3"
                          flag = 1
                      elif bond_length>V2_C3_N_CUTOFF and j.babel_type[0]=='N':
                          a.babel_type = "C3"
                          flag = 1
                      elif bond_length>V2_C3_O_CUTOFF and j.babel_type[0]=='O':
                          a.babel_type = "C3"
                          flag = 1
                      elif flag == 0:
                          if bond_length > GEN_C3_C_CUTOFF and \
                             j.babel_type[0] == 'C' and j.element == 'C':
                              a.babel_type = "C3"
                              flag = 1

    def phase5(self):

        for a in self.atoms:
            if a.babel_type == "C2":
                flag = 0;
                for b in a.bonds:
                    j = b.atom1
                    if j==a: j = b.atom2
                    if not j.babel_type in ["C3", "DC", "HC", "N3", "N3+", "O3"] and\
                       not j.babel_type in ["Pac", "Sac", "Sox", "C1", "S3", "Cac" ]:
                        flag = 1
                if flag == 0:
                    a.babel_type = "C3"


    def phase6(self):

        for a in self.atoms:
            no_plus = 1
            protonated = 1
            if a.babel_type=="N3":
                for b in a.bonds:
                    conn = b.atom1
                    if conn==a: conn = b.atom2
                    # If an unsaturated atom is attached to this nitrogen then
                    # it should be Npl
                    if len(a.bonds) == 2 and \
                       conn.babel_type in ["Car","C2","Sox","Sac","Pac","So2"]:
                        protonated = 0
                        a.babel_type = "Npl"

                    # If the attached atom is something other that C3,
                    # H or D the nitrogen is not positively charged
                    if conn.babel_type != "C3" and conn.babel_atomic_number != 1:
                        protonated = 0
                if protonated: a.babel_type = "N3+"

            # look for guanadinium nitrogens
            elif a.babel_type == "C2":
                # First see if we have an sp2 carbon surrounded by 3 sp2
                # nitrogens

                m = 0;
                for b in a.bonds:
                    k = b.atom1
                    if k==a: k = b.atom2
                    if k.babel_type=="Npl" or k.babel_type=="N2" or k.babel_type=="Ng+": m=m+1

                if m == 3:
                    a.babel_type = "C+"
                    for b in a.bonds:
                        k = b.atom1
                        if k==a: k = b.atom2
                        k.babel_type = "Ng+"

            elif a.babel_type == "Cac":
                for b in a.bonds:
                    k = b.atom1
                    if k==a: k = b.atom2
                    if k.babel_type[0]=="O" and self.count_heavy_atoms(k) == 1:
                        k.babel_type = "O-"


    def check_for_carbonyl(self, atom):
        for b in atom.bonds:
            bonded_atom = b.atom1
            if bonded_atom==atom: bonded_atom = b.atom2
            if bonded_atom.babel_type=="O2" or bonded_atom.babel_type=="S2":
                return 3
        return 2


    def check_for_amides(self):
  
        for a in self.atoms:
            if a.babel_type=="Npl":
                for b in a.bonds:
                    conn = b.atom1
                    if conn==a: conn = b.atom2
                    if conn.babel_type=="Cac" or conn.babel_type=="Sox" or \
                       conn.babel_type=="So2":
                        a.babel_type = "Nam"
                        break

                    if conn.babel_type=="C2":
                        if self.check_for_carbonyl(conn) == 3:
                            a.babel_type = "Nam"
                            break


    def getProperty(self, property, elements):
        """list <- getProperty(property, elements)

        property has to be 'num', 'cov_rad', 'bond_ord_rad', 'vdw_rad',
                           'bs_rad', 'max_bonds' or 'rgb'
        elements is a list of 1 or 2 character(s) strings
        """
        if property not in babel_elements["C"].keys():
            raise RuntimeError("Invalid property %s, has to be in %s\n" % \
                               (property, babel_elements["C"].keys()))
        prop = []
        for el in elements:
            prop.append(babel_elements[el][property])
        return prop

    
                               
class TypeConverter:

    def __init__(self, outputType):
        if outputType not in babel_types.keys():
            raise RuntimeError("Invalid format %s\n"%outputType)
        self.outputType = outputType


    def convert(self, input, mode='all_caps'):

        try:
            i = babel_types['INT'].index(input)
            return babel_types[self.outputType][i]
        except:
            print "Unable to assign %s type to atom %s"%(self.outputType,input)
            if mode=='zero':
                return 0
            elif mode=='dummy':
                i = babel_types['INT'].index("X")
                return babel_types[self.outputType][i]
            elif mode=='all_caps':
                return string.upper(input)
            else: return input

    def clean_atom_type(self, type_name):
       name = string.upper(type_name[0])
       if len(type_name) > 1:
           name = name + string.lower(type_name[1])
           if name[1] not in string.letters:
               return name[0]
       return name


if __name__ == '__main__':
    import pdb, sys
    from cycle import RingFinder
    from bo import BondOrder
    from aromatic import Aromatic
    
    from MolKit.pdbParser import NewPdbParser
    parser = NewPdbParser("/tsri/pdb/struct/%s.pdb"%sys.argv[1])
    mols = parser.parse()
    mol = mols[0]
    mol.buildBondsByDistance()
    allAtoms = mol.chains.residues.atoms
    bonds = allAtoms.bonds[0]

    print "assigning atom types"
    babel = AtomHybridization()
    babel.assignHybridization(allAtoms)




#############################################################################
#
# Author: Michel F. SANNER
# Reimplemented from Babel v1.6 from Pat Walters and Math Stahl
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/PyBabel/aromatic.py,v 1.3 2008/11/15 00:14:40 sargis Exp $
#
# $Id: aromatic.py,v 1.3 2008/11/15 00:14:40 sargis Exp $
#

"""
This file implements the Aromatic class.

Before this Aromatic object can be used to identify aromatic rings,
 1 - atoms must have been assigned a type see (AtomHybridization in types.py)
 2 - bond orders must have been defined
 3 - rings must have been identified (see RingFinder in cycle.py)

example:
    
      >>> atype = AtomHybridization()
      >>> atype.assignHybridization(atoms)
      >>> rings = RingFinder()
      >>> rings.findRings(atoms, bonds)
      >>> bo = BondOrder()
      >>> bo.assignBondOrder(atoms, bonds, rings)
      >>> arom = Aromatic(rings)
      >>> arom.find_aromatic_atoms(atoms)

      atoms has to be a list of atom objects
      Atom:
          a.coords : 3-sequence of floats
          a.bonds : list of Bond objects
          babel_type: string
          babel_atomic_number: int

      Bond:
          b.atom1 : instance of Atom
          b.atom2 : instance of Atom

      After completion
      
reimplmentation of Babel1.6 in Python by Michel Sanner April 2000
Original code by W. Patrick Walters and Matthew T. Stahl 
"""


import math

from atomTypes import TypeConverter
from util import distance, bond_angle

class Aromatic:
    """ """

    def __init__(self, rings):
        """ """
        self.rings = rings
        
    
    def find_aromatic_atoms(self, atoms):
        """ """
        info = self.setup_ring_info( atoms )

        for r in self.rings.rings:
            for a in r['atoms']:
                if a.arom_atm: # atom is aromatic
                    if a.babel_atomic_number == 6:
                        a.babel_type = "Car"
                    elif a.babel_atomic_number == 7:
                        a.babel_type = "Nar"

            for b in r['bonds']:
                if b.atom1.arom_atm and b.atom2.arom_atm:
                    b.bondOrder = 'aromatic'

        for r in self.rings.rings:
            for a in r['atoms']:
                if hasattr(a, 'arom_atm'): delattr(a, 'arom_atm')

                
    def find_aromatic_rings(self):
        """ """
        converter = TypeConverter("HYB")
       
        for r in self.rings.rings:
            is_aromatic = 0
            if len(r['atoms']) == 5 or len(r['atoms']) == 6:
                is_aromatic = 1
                for a in r['atoms']:
                    # check to see if the atom is sp2 
	            # if not, the ring isn't aromatic
                    hyb_str = converter.convert(a.babel_type, 'all_caps')
                    hyb = int(hyb_str)
                    if hyb != 2:
                        is_aromatic = 0
                        break

                    # check to see if the atom has 3 bonds to
                    # heavy atoms in the same ring.
                    # If not, the ring isn't aromatic

                    if self.count_arom_atm_bonds(a, r) != 3:
                        is_aromatic = 0
                        break
    
            if is_aromatic:
                r['aromatic'] = 1
                for a in r['atoms']: a.arom_atm = 1
                

    def find_aromatic_rings2(self):
        """ """
        converter = TypeConverter("HYB")
       
        for r in self.rings.rings:
            is_aromatic = 0
            if len(r['atoms']) == 5 or len(r['atoms']) == 6:
                is_aromatic = 1
                for a in r['atoms']:
                    # check to see if the atom is sp2 
                    # if not, the ring isn't aromatic

                    hyb_str = converter.convert(a.babel_type, 'all_caps')
                    hyb = int(hyb_str)
                    if hyb != 2:
                        is_aromatic = 0
                        break

                    # check to see if the atom has 3 bonds to
                    # heavy atoms in the same ring.
                    # If not, the ring isn't aromatic

#                    if not a.arom_atm:
#                        if self.count_arom_atm_bonds(a, r) != 3:
#                            is_aromatic = 0
#                            break
    
            if is_aromatic:
                r['aromatic'] = 1
                for a in r['atoms']: a.arom_atm = 1


    def setup_ring_info(self, atoms):
        """ """        
        for r in self.rings.rings:
            r['aromatic'] = 0
            for a in r['atoms']: a.arom_atm = 0
            
        self.find_aromatic_rings()
        self.find_aromatic_rings2()

        for r in self.rings.rings:
            # compute cycle center
            center = [0,0,0]
            sca = 1.0/len(r['atoms'])
            for a in r['atoms']:
                c = a.coords
                center[0] = center[0] + c[0]*sca
                center[1] = center[1] + c[1]*sca
                center[2] = center[2] + c[2]*sca
            r['center'] = center

            # compute cycle radius
            b = r['bonds'][0]
            c1 = b.atom1.coords
            c2 = b.atom2.coords
            mid = ( (c1[0]+c2[0])*0.5, (c1[1]+c2[1])*0.5, (c1[2]+c2[2])*0.5)
            r['radius'] = 0.9 * distance(center, mid)
            
            # compute cycle normal vector
            n_avg = [0,0,0]
            for i in range(len(r['bonds'])-1):
                b = r['bonds'][i]
                c1 = b.atom1.coords 
                c2 = b.atom2.coords 
                a3 = r['bonds'][i+1].atom1
                if a3==b.atom1 or a3==b.atom2: a3 = r['bonds'][i+1].atom2
                c3 = a3.coords
                v = (c2[0]-c1[0], c2[1]-c1[1], c2[2]-c1[2])
                v1 = (c3[0]-c1[0], c3[1]-c1[1], c3[2]-c1[2])
                n = [ v[1]*v1[2] - v1[1]*v[2],
                      v1[0]*v[2] - v[0]*v1[2],
                      v[0]*v1[1] - v1[0]*v[1] ]
                sca = 1.0 / math.sqrt(n[0]*n[0] + n[1]*n[1] + n[2]*n[2])
                n_avg = (n[0]*sca, n[1]*sca, n[2]*sca)

            r['normal'] = n_avg

            
    def count_arom_atm_bonds(self, atom, ring):
        """ """
        hvy_bonds = 0
        for b in atom.bonds:
            conn = b.atom1
            if conn==atom: conn=b.atom2
            if conn in ring['atoms']:
            
                if conn.babel_atomic_number != 1:
                    # add bond order of bonds to non hydrogen atoms
                    hvy_bonds = hvy_bonds+b.bondOrder

        return hvy_bonds


if __name__ == '__main__':
    import pdb, sys
    from atomTypes import AtomHybridization
    from cycle import RingFinder
    from bo import BondOrder
    from aromatic import Aromatic
    from addh import AddHydrogens
    
    from MolKit.pdbParser import PdbqParser
    #parser = NewPdbqParser("./txp_noh.pdbq") # doesn't work
    #parser = NewPdbqParser("./txp.pdbq")     # OK
    #parser = NewPdbqParser("./gmmtest.pdbq")  # 
    from MolKit.mol2Parser import NewMol2Parser
    #mols = parser.parse()
    #mol = mols[0]
    #mol.buildBondsByDistance()

    parser = NewMol2Parser("./mtx.mol2") # OK
    mols = parser.parse()
    mol = mols[0]

    allAtoms = mol.chains.residues.atoms

    print "assigning atom types"
    babel = AtomHybridization()
    babel.assignHybridization(allAtoms)

    print "looking for rings"
    bonds = allAtoms.bonds[0]
    rings = RingFinder()
    rings.findRings(allAtoms, bonds)

    print "assigning bond order"
    bo = BondOrder()
    #pdb.run("bo.assignBondOrder(allAtoms, bonds)")
    bo.assignBondOrder(allAtoms, bonds)

    def countMissingBonds(atoms):
        converter = TypeConverter('HYB')
        needDBatoms = []
        canHaveDBatoms = []
        for a in atoms:
            a._needsDB = 0
            a._canHaveDB = 0
            hyb = int(converter.convert(a.babel_type, 'all_caps'))
            if hyb != 2: continue         
            if a.babel_type[0] == 'C':
                sum = 0
                for b in a.bonds:
                    sum = sum + b.bondOrder

                if sum == 3:
                    if len(a.bonds)==3:
                        a._needsDB = 1
                        needDBatoms.append(a)
                    elif len(a.bonds)==2: # ==C-- or ==C/
                        a1 = a.bonds[0].atom1
                        if a1==a: a1 = a.bonds[0].atom2
                        a3 = a.bonds[1].atom1
                        if a3==a: a3 = a.bonds[1].atom2
                        ang = bond_angle( a1.coords, a.coords, a3.coords )
                        if math.fabs(ang-180) < 10: # ==C--
                            a._needDB = 1
                            needDBatoms.append(a)
                            
                elif sum == 2:
                    if len(a.bonds)==2:
                        a1 = a.bonds[0].atom1
                        if a1==a: a1 = a.bonds[0].atom2
                        a3 = a.bonds[1].atom1
                        if a3==a: a3 = a.bonds[1].atom2
                        ang = bond_angle( a1.coords, a.coords, a3.coords )
                        if math.fabs(ang-180) < 10:
                            a._needsDB = 2
                            needDBatoms.append(a)
                        else:
                            a._needsDB = 1
                            needDBatoms.append(a)

                elif sum == 1:
                    a._needsDB = 1
                    needDBatoms.append(a)
                    
            elif a.babel_type[0] == 'N':
                sum = 0
                for b in a.bonds:
                    sum = sum + b.bondOrder
                if sum == 2 and len(a.bonds)==2:
                    canHaveDBatoms.append(a)
                    a._canHaveDB = 1

            elif a.babel_type[0] == 'O':
                sum = 0
                for b in a.bonds:
                    sum = sum + b.bondOrder
                if sum == 1:
                    canHaveDBatoms.append(a)
                    a._canHaveDB = 1

        return needDBatoms, canHaveDBatoms


    def fixMissingBonds(pbatoms):
        i = 0
        while (len(pbatoms) and i <len(pbatoms)):
            atom = pbatoms[i]
            possibleNeighbors = []
            for b in atom.bonds:
                atom2 = b.atom1
                if atom2==atom: atom2=b.atom2
                if atom2._needsDB: possibleNeighbors.append( (atom2, b) )
            if len(possibleNeighbors)==atom._needsDB:
                for entry in possibleNeighbors:
                    b = entry[1]
                    a2 = entry[0]
                    b.bondOrder = b.bondOrder+1
                    a2._needsDB = a2._needsDB -1
                    atom._needsDB = atom._needsDB -1
                    if a2._needsDB==0: pbatoms.remove(a2)
                pbatoms.remove(atom)
            else:
                i = i +1
        return pbatoms


    def fixMissingBondOrders():
        needDB, canHave = countMissingBonds(allAtoms)
        while (1):
            l = len(needDB)
            needDB = fixMissingBonds(needDB)
            if len(needDB)==0: break
            if l==len(needDB): break

        print 'NEED'
        for a in needDB:
            print a
        print 'canHave'
        for a in canHave:
            print a
        all = []
##          while (1):
##              l = len(all)
##              all = fixMissingBonds(all)
##              if len(all)==0: break
##              if l==len(all): break
        return all
    
    pbatoms = fixMissingBondOrders()
    print "WARNING couldn't fix all bond orders", pbatoms

    print "looking for aromatic rings"
    arom = Aromatic(rings)
    arom.find_aromatic_atoms(allAtoms)
    print "done"

    import math

    def addDispVector():
        for b in bonds:                
            if b.bondOrder != 1:
                inring = hasattr(b.atom1, 'rings')
                c1 = b.atom1.coords
                c2 = b.atom2.coords
                if inring:
                    found = 0
                    for nb in b.atom1.bonds:
                        if found: break
                        if nb==b: continue
                        a3 = nb.atom1
                        if a3==b.atom1: a3 = nb.atom2
                        if hasattr(a3, 'rings'):
                            for ringNum in a3.rings:
                                if hasattr(b.atom1, 'rings'): r1=b.atom1.rings
                                else: r1 = []
                                if hasattr(b.atom2, 'rings'): r2=b.atom2.rings
                                else: r2 = []
                                if ringNum in r1 and ringNum in r2:
                                    found=1
                                    break
                else:
                    nb = b.atom1.bonds[0]
                    if nb==b: nb = b.atom2.bonds[0]
                    if nb==b:
                        if len(b.atom1.bonds)>1:
                            nb = b.atom1.bonds[1]
                        else:
                            nb = b.atom2.bonds[1]
                    a3 = nb.atom1
                    if a3==b.atom1: a3 = nb.atom2
                    
                c3 = a3.coords
                v = (c2[0]-c1[0], c2[1]-c1[1], c2[2]-c1[2])
                v1 = (c3[0]-c1[0], c3[1]-c1[1], c3[2]-c1[2])
                n = [ v[1]*v1[2] - v1[1]*v[2],
                      v1[0]*v[2] - v[0]*v1[2],
                      v[0]*v1[1] - v1[0]*v[1] ]
                nn = [ n[1]*v[2] - v[1]*n[2],
                       v[0]*n[2] - n[0]*v[2],
                       n[0]*v[1] - v[0]*n[1] ]
                sca = 0.15 / math.sqrt(nn[0]*nn[0] + nn[1]*nn[1] + nn[2]*nn[2])
                b.dispVec = (nn[0]*sca, nn[1]*sca, nn[2]*sca)

    addDispVector()
    
##      addh = AddHydrogens()
##      hat = addh.addHydrogens(allAtoms)
    
##      from MolKit.molecule import Atom, Bond
##      for a in hat:
##          atom = Atom('H', a[1].parent, top=a[1].top)
##          atom._coords = [ a[0] ]
##          if hasattr(atom, 'segID'):
##              atom.segID = a[1].segID
##          atom.hetatm = 0
##          atom.alternate = []
##          atom.element = 'H'
##          atom.number = -1
##          atom.occupancy = 1.0
##          atom.conformation = 0
##          atom.temperatureFactor = 0.0
##          atom.babel_atomic_number = a[2]
##          atom.babel_type = a[3]
##          atom.babel_organic=1
##          bond = Bond( a[1], atom )

    from Pmv.moleculeViewer import MoleculeViewer
    mv = MoleculeViewer()
    mv.addMolecule(mol)
    mol.bondsflag = 1
    mv.lines( mol )

    v = []
    l = []
    g = mv.Mols[0].geomContainer.geoms['lines']
    vc = len(g.vertexSet)
    scale = [0, 1, -1, 2, -2, 3, -3, 4, -4]
    add = -1

##      for b in bonds:
##          if b.bondOrder>1:
##              b.bondOrder = 5
##              break

    for b in bonds:
        n = b.bondOrder
        if n=='aromatic': continue
        for i in range(1, n):
            print i, scale[i]
            l.append( (vc,vc+1) )
            c1 = b.atom1.coords
            v.append( c1[0]+(scale[i]*b.dispVec[0]),
                      c1[1]+(scale[i]*b.dispVec[1]),
                      c1[2]+(scale[i]*b.dispVec[2]) )
            vc = vc+1
            c1 = b.atom2.coords
            v.append( c1[0]+(scale[i]*b.dispVec[0]),
                      c1[1]+(scale[i]*b.dispVec[1]),
                      c1[2]+(scale[i]*b.dispVec[2]) )
            vc = vc+1

    g = mv.Mols[0].geomContainer.geoms['lines']
    g.Add(vertices = v, faces=l)

    from DejaVu.Arcs3D import Arcs3D
    aromC = []
    aromR = []
    aromN = []
    for r in rings.rings:
        if r['aromatic']:
            aromC.append(r['center'])
            aromR.append(r['radius'])
            aromN.append(r['normal'])

    c = Arcs3D('aromCircles', vertices = aromC, vnormals = aromN,
                materials = ( (0,1,0), ), radius = aromR,
                angles = ( 360, ), stippleLines = 1, lineWidth=2)

    mv.Mols[0].geomContainer.geoms['aromCircles'] = c
    mv.GUI.VIEWER.AddObject(c, parent = mv.Mols[0].geomContainer.masterGeom)



#############################################################################
#
# Author: Michel F. SANNER
# Reimplemented from Babel v1.6 from Pat Walters and Math Stahl
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/PyBabel/tools.py,v 1.1.1.1 2001/04/03 19:47:47 gillet Exp $
#
# $Id: tools.py,v 1.1.1.1 2001/04/03 19:47:47 gillet Exp $
#
#
#

import string

def read_element_table(filename):
    """void <- read_element_table(filename)
    populates the elementsTable dictionary from the a given file.
    the file provides:
    line number, element string, cov_rad, bond_ord_rad, vdw_rad, bs_rad,
    max_bonds, red, green, blue
    """
    f = open(filename)
    lines = f.readlines()
    f.close()
    elemTable = {}
    for i in range(len(lines)):
        dd = string.split(lines[i])
        elemTable[dd[1]] = { 'num':i,
                             'cov_rad':float(dd[2]),
                             'bond_ord_rad':float(dd[3]),
                             'vdw_rad':float(dd[4]),
                             'bs_rad':float(dd[5]),
                             'max_bonds':int(dd[6]),
                             'rgb': (float(dd[7]),float(dd[8]),float(dd[9]))
                           }
    return elemTable


def writeElementTableAsPythonCode(elemTab, inFileName, outFileName):
    """write elemTable as a python dictionary that can be imported"""

    f = open(outFileName,'w')
    f.write("# File generated from %s\n#\n"%inFileName)
    f.write("babel_elements = {\n")
    for k,v in elemTab.items():
        f.write("  '%s': %s, \n" % (k,str(v)))
    f.write('}\n#END\n');
    f.close()


def read_types_table(filename):
    f = open(filename)
    typestab = {}
    nrow, ncol = map( int, string.split(f.readline()))
    typeFormats = string.split(f.readline())
    for t in typeFormats:
        typestab[t] = []
    for i in range(nrow-1):
        typeNames = string.split(f.readline())
        for j in range(ncol):
            typestab[typeFormats[j]].append(typeNames[j])
    f.close()
    return typestab


def writeTypesTableAsPythonCode(typestab, inFileName, outFileName):
    """write typestab as a python dictionary that can be imported"""

    f = open(outFileName,'w')
    f.write("# File generated from %s\n#\n"%inFileName)
    f.write("babel_types = {\n")
    for k,v in typestab.items():
        f.write("  '%s': %s, \n" % (k,str(v)))
    f.write('}\n#END\n');
    f.close()


if __name__ == '__main__':
    # write tables
    et = read_element_table('element.lis')
    writeElementTableAsPythonCode(et, 'element.lis', 'babelElements.py')

    tt = read_types_table('types.lis')
    writeTypesTableAsPythonCode(tt, 'types.lis', 'babelAtomTypes.py')



#############################################################################
#
# Author: Michel F. SANNER
# Reimplemented from Babel v1.6 from Pat Walters and Math Stahl
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/PyBabel/bo.py,v 1.4 2002/10/22 23:11:38 sanner Exp $
#
# $Id: bo.py,v 1.4 2002/10/22 23:11:38 sanner Exp $
#

"""
This file implements the BondOrder class that can be used to compute
bond order.

Before a BondOrder object can be used, atoms must have been assigned
a type see (AtomHybridization in types.py).

Bond order can be calculated using 2 different methods depending on whether
rings have been identified previously or not. Babel decides to use the first
method for molecules with more than 200 atoms and the second one else.
    
example:
      >>> from PyBabel.atomTypes import AtomHybridization
      >>> from PyBabel.cycle import RingFinder
      >>> atype = AtomHybridization()
      >>> atype.assignHybridization(atoms)
      >>> bo = BondOrder()
      >>> bo.assignBondOrder( atoms, bonds )

      or

      >>> atype = AtomHybridization()
      >>> atype.assignHybridization(atoms)
      >>> rings = RingFinder()
      >>> rings.findRings(allAtoms, bonds)
      >>> bo = BondOrder()
      >>> bo.assignBondOrder( atoms, bonds, rings )

      atoms has to be a list of atom objects
      Atom:
          a.coords : 3-sequence of floats
          a.bonds : list of Bond objects
          babel_type: string
          babel_atomic_number: int

      Bond:
          b.atom1 : instance of Atom
          b.atom2 : instance of Atom

      after completion each bond has a 'bondOrder' attribute (integer)

reimplmentation of Babel1.6 in Python by Michel Sanner April 2000
Original code by W. Patrick Walters and Matthew T. Stahl 
"""



import string, math

from babelAtomTypes import babel_types
from babelElements import babel_elements
from util import *
from atomTypes import TypeConverter


SINGLE_DOUBLE_CUTOFF = 0.95
#SINGLE_DOUBLE_CUTOFF = 0.955
DOUBLE_TRIPLE_CUTOFF = 0.81

class BondOrder:
    """ """

    def assignBondOrder(self, atoms, bonds, rings=None):
        """ """
        if not rings:
            self.assign_bond_order1(atoms, bonds)
        else:
            self.assign_bond_order2(atoms, bonds, rings)


    def assign_bond_order1(self, atoms, bonds):
        """ """

        hyb_val = [0,3,2,1]
        converter = TypeConverter("HYB")
        for a in atoms:
            hyb = converter.convert(a.babel_type, 'dummy')
            a._redo = hyb_val[int(hyb)]
            #print a.full_name(), a.babel_type, hyb, a._redo

        for b in bonds:
            # initialize bondOrder attribute
            if b.bondOrder is None:
                b.bondOrder = 1
            sum_code = b.atom1._redo + b.atom2._redo
            #print b, sum_code
            if sum_code == 6:
                b.bondOrder = 3
            elif sum_code == 4:
                b.bondOrder = 2
            else:
                b.bondOrder = 1
            
            if self.is_carboxyl(b):
                b.bondOrder = 2

            if b.bondOrder < 1 or b.bondOrder > 3:
                print "Bond %s is wierd - Bond order is %d\n" % \
                      (b, b.bondOrder)

        self.check_for_conjugation(atoms)

        # cleanup
        for a in atoms:
            delattr(a, '_redo')


    def is_carboxyl(self, bond):
        """ """
        c_end = 0
        o_end = 0
        check = 0
        
        if bond.atom1.babel_type == "Cac" and bond.atom2.babel_type == 'O':
            c_end = bond.atom1
            o_end = bond.atom2
            check = 1

        if bond.atom2.babel_type == "Cac" and bond.atom1.babel_type == 'O':
            check = 1
            c_end = bond.atom2
            o_end = bond.atom1

        if check and len(o_end.bonds) == 1: return 1
        else: return 0


    def check_for_conjugation(self, atoms):
        """ """

        for a in atoms:
            #if a.full_name()=='1crn: :ASN14:CG': raise
            for b1 in a.bonds:
                if b1.bondOrder<=1: continue
                for b2 in a.bonds:
                    if b1==b2: continue
                    if b2.bondOrder<=1: continue
                    if len(b2.atom1.bonds) > 1 and len(b2.atom2.bonds) > 1:
                        b2.bondOrder = 1


    def check_for_carbonyl(self, atom):
        """ """
        for b in atom.bonds:
            bonded_atom = b.atom1
            if bonded_atom==atom:
                bonded_atom = b.atom2
            if bonded_atom.babel_type=="O2" or bonded_atom.babel_type=="S2":
                return 3
            return 2


    def assign_bond_order2(self, atoms, bonds, rings):
        """ """

        self.rings = rings

        for a in atoms:
            if hasattr(a, 'rings'):
                a._redo = 1
            else:
                a._redo = 0
            a._dot = 0
            a._dbatom = 0
            
        self.assign_hybrid_radii(atoms)
        self.estimate_bond_order2(bonds)

        for ring in self.rings.rings:
            if len(ring['atoms'])==5:
                self.process_5_ring(ring)

        for b in bonds:
            # initialize bondOrder attribute
            if b.bondOrder is None:
                b.bondOrder = 1
            b._dbbond = 0
            if b.bondOrder == 2:
                if len(b.atom1.bonds) > 1 and b.atom1.babel_type[0] == 'O':
                    b.bondOrder = 1
                elif len(b.atom2.bonds) > 1 and b.atom2.babel_type[0] == 'O':
                    b.bondOrder = 1

        for b in bonds:
            if b.bondOrder == 2:
                if len(b.atom1.bonds) == 1 and b.atom1.babel_type[0] == 'N':
                    b.bondOrder = 1
                elif len(b.atom2.bonds) == 1 and b.atom2.babel_type[0] == 'N':
                    b.bondOrder = 1

##          for b in bonds:
##              if b.bondOrder > 1:
##                  print "%3d %3d"%(atoms.index(b.atom1)+1, atoms.index(b.atom2)+1)
        for b in bonds:
            if b.bondOrder > 1:
                a1 = b.atom1
                if a1._redo and self.check_for_carbonyl(a1) != 3:
                    a1._dot = 1
                a2 = b.atom2
                if a2._redo and self.check_for_carbonyl(a2) != 3:
                    a2._dot = 1
                if len(a1.bonds) == 1 or len(a2.bonds) == 1:
                    a1._dot = 0
                    a2._dot = 0

##          for a in atoms:
##              if a._dot==1:
##                  print atoms.index(a)+1, a._dot

        for a in atoms:
            if a.babel_type=="Npl" and len(a.bonds) == 3:
                a._dot = 0

##          for a in atoms:
##              if a._dot==1:
##                  print atoms.index(a)+1, a._dot

        self.atoms = atoms
        self.cycles = 0
        self.bondStack = []
        self.bondChoice = []

        # for PYTHON interpreters after 1.5 the recursion depth is limited
        # this method can exceed the default recursion depth
        import sys
        if float(sys.version[:3]) > 1.5:
            sys.setrecursionlimit(20000)

        self.connect_the_dots(0,0)
        
        for b in self.bondStack:
            if b.bondOrder > 1:
                b._dbbond = 1
                b.atom1._dbatom = 1
                b.atom2._dbatom = 1

        for b in bonds:
            if b.atom1.babel_type=="O2" or b.atom2.babel_type=="O2":
                b._dbbond = 1
                b.atom1._dbatom = 1
                b.atom2._dbatom = 1
            elif b.atom1.babel_type=="O-" and len(b.atom1.bonds) == 1:
                b._dbbond = 1
                b.atom1._dbatom = 1
                b.atom2._dbatom = 1

            elif b.atom2.babel_type=="O-" and len(b.atom2.bonds) == 1:
                b._dbbond = 1
                b.atom1._dbatom = 1
                b.atom2._dbatom = 1

        for a in atoms:
            a._dot = 0

        for b in bonds:
            if b.bondOrder > 1:
                a1 = b.atom1
                a2 = b.atom2
                if a1._dbatom==0 and a2._dbatom==0:
                    a1._dot = 1
                    a2._dot = 1

        self.bondStack = []
        self.bondChoice = []
        self.connect_the_dots(0,0)

        for b in self.bondStack:
            if b.bondOrder > 1:
                b._dbbond = 1

##          for b in bo.bondStack:
##              a1 = b.atom1
##              a2 = b.atom2
##              print a1.number, a2.number, a1._dot, a2._dot, b.bondOrder


        for b in self.bondStack:
            b._dbbond = 1

        for b in bonds:
            if not b._dbbond:
                b.bondOrder = 1

        for a in atoms:
            a._redo = 0
            for b in a.bonds:
                a._redo = a._redo + b.bondOrder

            if (a.babel_atomic_number == 6 or a.babel_atomic_number) == 7 and \
               a._redo > 4:

                for b in a.bonds:
                    conn=None
                    if b.bondOrder == 2:
                        b.bondOrder = 1

        #cleanup
        for a in atoms:
            delattr(a, '_dot')
            delattr(a, '_dbatom')
            delattr(a, '_redo')
            
        for b in bonds:
            delattr(b, '_dbbond')
            
        delattr(self, 'atoms')
        delattr(self, 'cycles')
        delattr(self, 'rings')

##          for b in bonds:
##              n = b.atom1.number
##              print "%4i %4i %2i"%(n, b.atom2.number, b.bondOrder)


    def connect_the_dots(self, atom, start):
        """ """
        a = self.atoms[atom]
        if start == len(a.bonds): return

        #print 'AAA', atom+1

        if a._dot:
            done = 0
            i = start
            for b in a.bonds[start:]:
                con = b.atom1
                if con==a: con=b.atom2
                if con._dot:
                    self.bondStack.append(b)
                    self.bondChoice.append(0)
                    if a==b.atom1:
                        self.bondChoice[-1] = i+1
                    else:
                        self.bondChoice[-1] = -i-1
                    a._dot = a._dot-1
                    con._dot = con._dot-1
                    done = 1
                    break
                i = i + 1
                
            if not done and len(self.bondStack):
                b = self.bondStack[-1]
                if self.bondChoice[-1] > 0: new_atm = b.atom1
                else: new_atm = b.atom2
                choice_bnd = abs(self.bondChoice[-1])
                self.bondChoice = self.bondChoice[:-1]
                self.bondStack = self.bondStack[:-1]
                b.atom1._dot = b.atom1._dot + 1
                b.atom2._dot = b.atom2._dot + 1
                #print 'BBB', self.atoms.index(new_atm)+1
                self.connect_the_dots(self.atoms.index(new_atm), choice_bnd )

        if self.cycles > 10000:
           #print 'EEE'
           return

        if atom+1 == len(self.atoms):
           #print 'DDD'
           return
        else:
            self.cycles = self.cycles+1
            #print 'CCC', atom+2
            self.connect_the_dots(atom+1,0)


    def get_bond(self, bonds, a1, a2):
        """ """
        for b in bonds:
            if (b.atom1==a1 and b.atom2==a2) or (b.atom1==a2 and b.atom2==a1):
                return b


    def process_5_ring(self, ring):
        """ """
        atoms = ring['atoms']
        t1 = torsion_angle(atoms[4].coords,atoms[0].coords,
                           atoms[1].coords,atoms[2].coords)
        t2 = torsion_angle(atoms[0].coords,atoms[1].coords,
                           atoms[2].coords,atoms[3].coords)
        t3 = torsion_angle(atoms[1].coords,atoms[2].coords,
                           atoms[3].coords,atoms[4].coords)
        t4 = torsion_angle(atoms[2].coords,atoms[3].coords,
                           atoms[4].coords,atoms[0].coords)
        t5 = torsion_angle(atoms[3].coords,atoms[4].coords,
                           atoms[0].coords,atoms[1].coords)

        if math.fabs(t1) < 7.0:
            a1 = atoms[0]
            a2 = atoms[1]
            bond = self.get_bond(ring['bonds'], a1, a2 )
            bond.bondOrder = 1
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            if ratio < SINGLE_DOUBLE_CUTOFF:
                bond.bondOrder = 2

        if math.fabs(t2) < 7.0:
            a1 = atoms[1]
            a2 = atoms[2]
            bond = self.get_bond(ring['bonds'], a1, a2 )
            bond.bondOrder = 1
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            if ratio < SINGLE_DOUBLE_CUTOFF:
                bond.bondOrder = 2

        if math.fabs(t3) < 7.0:
            a1 = atoms[2]
            a2 = atoms[3]
            bond = self.get_bond(ring['bonds'], a1, a2 )
            bond.bondOrder = 1
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            if ratio < SINGLE_DOUBLE_CUTOFF:
                bond.bondOrder = 2

        if math.fabs(t4) < 7.0:
            a1 = atoms[3]
            a2 = atoms[4]
            bond = self.get_bond(ring['bonds'], a1, a2 )
            bond.bondOrder = 1
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            if ratio < SINGLE_DOUBLE_CUTOFF:
                bond.bondOrder = 2

        if math.fabs(t5) < 7.0:
            a1 = atoms[4]
            a2 = atoms[0]
            bond = self.get_bond(ring['bonds'], a1, a2 )
            bond.bondOrder = 1
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            if ratio < SINGLE_DOUBLE_CUTOFF:
                bond.bondOrder = 2


    def estimate_bond_order2(self, bonds):
        """ """
        converter = TypeConverter("HYB")
        for b in bonds:
            bo = 1
            a1 = b.atom1
            a2 = b.atom2
            dist = distance(a1.coords, a2.coords)
            cov_sum = a1.babel_bond_ord_rad + a2.babel_bond_ord_rad
            ratio = dist/cov_sum
            start_type = converter.convert(a1.babel_type, "all_caps")
            end_type = converter.convert(a2.babel_type, "all_caps")

            if ratio <= DOUBLE_TRIPLE_CUTOFF:
                if start_type[0] == '1' and end_type[0] == '1':
                    bo = 3
            elif ratio <= SINGLE_DOUBLE_CUTOFF:
                if start_type[0] == '2' and end_type[0] == '2':
                    bo = 2
            b.bondOrder = bo
       
        
    def assign_hybrid_radii(self, atoms):
        """ """
        converter = TypeConverter("XYZ")
        for a in atoms:
            atm_type = converter.convert(a.babel_type, 'zero')
            if atm_type == 0:
                atm_type = a.babel_type
            atm_type = converter.clean_atom_type(atm_type)

            a.babel_cov_rad = babel_elements[atm_type]['cov_rad']
            a.babel_bond_ord_rad = babel_elements[atm_type]['bond_ord_rad']
            a.babel_max_bonds = babel_elements[atm_type]['max_bonds']


if __name__ == '__main__':
    import pdb, sys
    from atomTypes import AtomHybridization
    from cycle import RingFinder
    
    from MolKit.pdbParser import NewPdbParser
    parser = NewPdbParser("/tsri/pdb/struct/%s.pdb"%sys.argv[1])
    mols = parser.parse()
    mol = mols[0]
    mol.buildBondsByDistance()
    allAtoms = mol.chains.residues.atoms
    bonds = allAtoms.bonds[0]

    print "assigning atom types"
    babel = AtomHybridization()
    babel.assignHybridization(allAtoms)

    print "looking for rings"
    rings = RingFinder()
    rings.findRings(allAtoms, bonds)

    print "assigning bond order"
    bo = BondOrder()
    #pdb.run("bo.assignBondOrder(allAtoms, bonds)")
    bo.assignBondOrder(allAtoms, bonds)

    for b in bonds:
        if b.bondOrder > 1:
            a1 = b.atom1
            a2 = b.atom2
            print '%-20s %-20s %d'% ( a1.full_name(), a2.full_name(),
                                      b.bondOrder )
